<?php
/**
 * Created by JetBrains PhpStorm.
 * User: geovanesantos
 * Date: 08/10/14
 * Time: 14:18
 * To change this template use File | Settings | File Templates.
 */

namespace Scheduler\Facade;


/**
 * Class ProcessParamChecker
 * Algumas funções que servem para checar e validar parâmetros de um processo do Scheduler.
 * @package Scheduler\Facade
 */
class ProcessParamChecker {
    /**
     * Checa os campos na tela de parâmetros de processo do sistema Scheduler
     * @param bool $isWizard
     *  Se está sendo executado do Wizard. Serve para evitar conflitos de variável de ambiente entre o wizard
     *  e a tela normal
     *
     * @return bool
     *  true se todos os valores estão válidos, false se não
     */
    public static function checkFields($isWizard=false){
        if(!$isWizard){
            $activeFields=tk_get_env_var("ACTIVE_FIELDS"); //pega os campos que estão ativos, só vai checar esses
        }
        else{
            $activeFields=tk_get_env_var("ACTIVE_FIELDS_WIZARD");
        }

        $regexEmail="/[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,4})/"; //regex p/ emails

        $filledCheck=true;  //todos os campos preenchidos
        $emailCheck=true;   //todos os e-mails corretos
        $paramCheck=true;   //todos os parâmetros corretos

        foreach($activeFields as $field){          // para cada campo
            if($field==="RECIPIENT"){     //se for recipient tem que checar se tem e se a sintaxe de todos tá certa
                $arrayAddresses=explode(",",tk_get_field_value("RECIPIENT"));  //e-mails separados por vírgula
                if (tk_get_field_value($field)==="" AND tk_get_field_value("CONTACTS")==="N"){
                    $filledCheck=false;  //está vazio
                }
                foreach ($arrayAddresses as $address){ //para cada e-mail
                    if (preg_match($regexEmail,$address)==0) $emailCheck=false; //se a sintaxe está errada
                }
            }

            // se o campo for parameters (de procedure) checa se os parâmetros estão certos
            else if ($field==="PARAMETERS"){

                //NOTA: uma procedure tem seus próprios parâmetros
                //pega valores esperados pela procedure escolhida

                $parameterName=tk_get_field_value("PROCEDURE_NAME");
                $currentProcedureParams=\Scheduler\Util\ProcedureParameterFunctions::getProcedureParamsAsArray($parameterName);


                $value=tk_get_field_value($field);//pega o valor do campo
                $attribs=explode("\n",$value);  //separa cada um dos parâmetros (em PARAMETERS), um por linha

                //explode volta um array com "" se receber ""
                //tem que converter p/ array (p/ o caso de uma procedure sem parametros)
                if(empty($attribs[0])) $attribs=Array();

                if(count($attribs) == count($currentProcedureParams)){ //se o nº de params fornecidos=requeridos

                    foreach($attribs as $attrib){
                        $attribArray=explode("=", $attrib);//separa nome do valor

                        //checa se o nome do parâmetro existe nos requeridos e se o valor não é vazio
                        @$currentParamType=$currentProcedureParams[$attribArray[0]];
                        $paramCheck=$paramCheck && isset($currentParamType);
                        $paramCheck=$paramCheck && self::checkParamType($attribArray[1],$currentParamType);
                    }
                } else $paramCheck=false; // se o nº não estiver igual está errado

            }

            //se não for nenhum dos dois e não for attachment, não pode estar vazio
            else if (tk_get_field_value($field)==="" && ($field!=="ATTACHMENT")) $filledCheck=false;
        }
        if ($filledCheck===false) tk_warning("Há campos não preenchidos!");
        if ($emailCheck===false) tk_warning("Há um e-mail de destinatário inválido!");
        if ($paramCheck===false){
            tk_warning("Alguns parâmetros estão incorretos!");
//            \Scheduler\Facade\ParameterFieldManager::loadParameters();
        }
        return ($filledCheck AND $emailCheck AND $paramCheck);
    }

    /**
     * Valida o valor do parâmetro em $param, vendo se é do tipo $type. Usado para validar os parâmetros de uma
     * procedure, passados pelo campo "PARAMETERS". Se $param não for do tipo esperado, tenta executar $param como
     * se fosse uma função do banco de dados e checa se o retorno dessa é do tipo esperado.
     *
     * @param $param
     *  O valor a ser validado
     * @param $type
     *  O tipo. Um número, que equivale a um tipo de dado SQL como descrito nas constantes classe
     *  \Setforms\Runtime\Helper\Procedure\Param.
     * @return bool
     *  true se é válido, false se não. Note que se $param for uma função de banco e retornar o tipo esperado,
     *  a função retornará true também
     */
    private static function checkParamType($param, $type){

        switch($type){
            case "1":
               return \Scheduler\Facade\ProcessParamChecker::checkInteger($param);
            case "6":
               return \Scheduler\Facade\ProcessParamChecker::checkDate($param);
            default:
                return \Scheduler\Facade\ProcessParamChecker::checkString($param);

        }
    }

    /**
     * Executa ProcessParamChecker::verifyInput com um callback que checa se $param representa um número
     * @param $param
     * @return bool
     */
    public static function checkInteger($param){

        return ProcessParamChecker::verifyInput('is_numeric', $param);
    }

    /**
     * Executa ProcessParamChecker::verifyInput com um callback que checa se $param representa uma data
     *
     * @param $param
     * @return bool
     */
    public static function checkDate($param){

        return ProcessParamChecker::verifyInput(Array('\Scheduler\Facade\ProcessParamChecker', 'isDate'), $param);
    }

    /**
     * Verifica somente se $param é uma string válida que não seja
     * a string indicativa mostrada ao escolher uma procedure.
     *
     * @param $param
     * @return bool
     */
    public static function checkString($param){
        if (!ProcessParamChecker::isntPlaceholder($param)) return false;
        if (!strpos($param, "'")&&((strrpos($param,"'")!==(strlen($param)-1)) || (strpos($param,"'")!==0))) return false;
        else{
            $db = \Setforms2\Runtime\Connection::getInstance();
            try{
                if (ProcessParamChecker::isDBSafe($param)){
                    //executando o parâmetro para checar se é uma função
                    $db->fetchAll("SELECT ".$param. " FROM dual");
                    return true;
                } else return false;
            }
            catch (\Exception $e){
                tk_warning("A função passada como parâmetro é inválida.");
                return false;
            }
        }
    }

    /**
     * Verifica se $input não é a string indicativa mostrada ao escolher uma procedure.
     * @param $input
     * @return bool
     */
    public static function isntPlaceholder($input){
        //Checa se a string dada não começa e termina com chaves
        return (strpos($input, "{")!==1) || (strpos($input, "}")!==(strlen($input)-2));
    }

    /**
     * Verifica se $input é uma data válida
     * @param $input
     * @return bool
     */
    public static function isDate($input){
        $trimmedInput=trim($input, "\"");
        if(strtotime($trimmedInput)!==false){
            return true;
        }
        else return false;
    }

    /**
     * Verifica se $param ou retorna true como parâmetro de $callback, ou é uma função de banco cujo resultado
     * retorna true quando parâmetro de $callback
     *
     * @param callable $callback
     *  Callback contra o qual $param será testado. Note que callback TEM que retornar true ou false
     *  ao receber $param e, se esse resultado for false, também TEM que retornar true ou false
     *  ao receber o resultado da execução de $param no banco como parâmetro
     *
     * @param $param
     * @return bool
     *  true se
     */
    private static function verifyInput($callback, $param){
        if (call_user_func($callback,$param)) return true; //se o callback validar $param retorna aqui
        else{
            $db = \Setforms2\Runtime\Connection::getInstance();
            try{
                //verifica se $param não tem algum drop, alter, e outras coisas que
                //não devem ser executadas aqui
                if(ProcessParamChecker::isDBSafe($param)){
                    //se é seguro, executa pra ver o resultado
                    $result=$db->fetchAll('SELECT '.$param.' FROM dual');

                    /* Note que fetchAll retorna uma matriz, [row][column]
                     * que (se espera) tem um dado só
                     * para pegar o dado, usa-se dois array_pop em sequência
                     */
                    if (call_user_func($callback,array_pop(array_pop($result)))) return true;
                    else return false;
                } else return false;
            }
            catch (\Exception $e){
                tk_warning("A função passada como parâmetro é inválida.");
                return false;
            }
        }
    }

    /**
     * Verifica se $inputToCheck tem keywords que alterariam dados ou a estrutura da base
     * (no caso, drop, alter e update)
     *
     * @param $inputToCheck
     * @return bool
     *  true se não tem, false se tem
     */
    public static function isDBSafe($inputToCheck){

        //remove possíveis comentários que escondam as keywords procuradas
        //e.g. dr/**/op
        $inputToCheck=str_replace('/*', "", $inputToCheck);
        $inputToCheck=str_replace('*/', "", $inputToCheck);

        //explode o input em '
        //a idéia é que o array resultante vai ter em suas posições pares as partes do input que não estão
        //encapsuladas em aspas
        //por exemplo, "isso é 'uma' string" terá como pos. 0 "isso é ", pos. 1 "uma" e pos. 2 " string"
        $inputExploded=explode("'",$inputToCheck);

        $hasDrop=true;
        $hasAlter=true;
        $hasUpdate=true;
        //aqui só se checa as posições pares, com um for que pula de dois em dois
        for($i=0; $i<=count($inputExploded); $i+=2){
            //modifica a flag se achar a string procurada
            $hasDrop = $hasDrop && (stripos($inputExploded[$i], 'drop')===false);
            $hasAlter = $hasAlter && (stripos($inputExploded[$i], 'alter')===false);
            $hasUpdate = $hasUpdate && (stripos($inputExploded[$i], 'update')===false);
        }

        return $hasDrop && $hasAlter && $hasUpdate;
    }
}