import {
  IEventParam, Loader, IDictionary, Metadata,
} from '@zeedhi/core';

import {
  Button, ModalService, Modal, IGridEvent, Grid, SelectMultiple, Select,
} from '@zeedhi/common';

export class RelatoriosEmissaoCardController {
  public utilities = Loader.getInstance('Utilities');
  
  public appController = Loader.getInstance('AppController');
  
  private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');
  
  public dtIniCard = '';

  public dtFinCard = '';
  
  public dtIniPeriodo = '';

  public dtFinPeriodo = '';

  public descCard = '';
  
  private cdtipoCard = '';

  private paramsCard?: IDictionary;

  private gridPeriodo!: Grid;
  
  private modalEmissao!: Modal;
  
  private modalPeriodoEmissao!: Modal;
  
  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }
  
  public async createModalEmissao() {
    if (!this.modalEmissao) {
      this.modalEmissao = await ModalService.createFromJsonPost('',
        { jsonPath: 'pme/relatorios/modals/emissaoCardFrame' });
    } else {
      // this.modalEmissao.show();
      await this.modalEmissao.destroy();
      this.modalEmissao = await ModalService.createFromJsonPost('',
        { jsonPath: 'pme/relatorios/modals/emissaoCardFrame' });
    }
  }
  
  public async cancelModalEmissao() {
    if (this.modalEmissao) {
      await this.modalEmissao.destroy();
      this.modalEmissao = this.utilities.modalEmpty;
    } else {
      const modal = ModalService.getModal('modalEmissaoCard');
      await modal?.destroy();
      this.modalEmissao = this.utilities.modalEmpty;
    }
  }
  
  public showModalEmissao({ component }: IEventParam<Button>) {
    this.createModalEmissao().then(() => {
      let namePath = '';
      
      if (component.name === 'cardDia') namePath = '/pme/relatorios/pme03100_emissao_card_dia';
      else if (component.name === 'cardSemana') namePath = '/pme/relatorios/pme01000_emissao_card_semana';
      else namePath = '/pme/relatorios/pme03000_emissao_card_periodo';
      
      this.modalEmissao.children[1].params.jsonPath = namePath;
      this.modalEmissao.show();
    });
  }
  
  public setfullscreenEmissao({ component }: IEventParam<Button>) {
    this.modalEmissao.fullscreen = !this.modalEmissao.fullscreen;
    this.modalEmissao.cssClass = this.modalEmissao.fullscreen ? 'fullscreen-fill-content' : 'modal-emissao-cardapio';
    
    component.iconName = (this.modalEmissao.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
  }
  
  public async createPeriodoModal() {
    this.modalPeriodoEmissao = await ModalService.createFromJsonPost('',
      { jsonPath: 'pme/relatorios/modals/gridPeriodoEmissao' });
  }

  public destroyPeriodoModal() {
    if (this.modalPeriodoEmissao) {
      this.modalPeriodoEmissao.destroy();
      this.modalPeriodoEmissao = this.utilities.modalEmpty;
    }
  }
  
  public closeModal() {
    this.modalPeriodoEmissao.hide();
  }
  
  public clearFields() {
    const serviceElement = Metadata.getInstance<SelectMultiple>('SERVICO');
    
    serviceElement.setValue([]);
    this.dtIniCard = '';
    this.dtFinCard = '';
    this.dtIniPeriodo = '';
    this.dtFinPeriodo = '';
    this.descCard = '';
  }
  
  public setCdTipoCard(cdtipoCard: IDictionary) {
    this.cdtipoCard = (typeof cdtipoCard === 'object') ? cdtipoCard.CDTIPOCARD : cdtipoCard;
  }
  
  public setGridInstance({ component }: IGridEvent) {
    this.gridPeriodo = component;
  }
  
  public setFilterModalPeriodo() {
    if (this.gridPeriodo !== undefined) {
      const conditions = [
        {
          name: 'P_CDFILCENTPME',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
        {
          name: 'P_CDTIPOCARD',
          operator: '=',
          value: this.cdtipoCard,
        },
      ];

      this.gridPeriodo.datasource.filter = conditions;
      this.gridPeriodo.datasource.get();
    }
  }
  
  public setPeriodoParams({ row }: IGridEvent) {
    const modal = this.gridPeriodo.parent as Modal;
    
    this.paramsCard = row;
    this.dtIniCard = row.DTINICARD;
    this.dtFinCard = row.DTFINCARD;
    this.dtIniPeriodo = row.DTINICARD;
    this.dtFinPeriodo = row.DTFINCARD;
    this.descCard = row.DSCARDPME;
    
    modal.hide();
  }
  
  public getParamsCard() {
    return this.paramsCard;
  }
  
  public openModalSearch() {
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD');
    
    if (tipoCard.value) {
      this.modalPeriodoEmissao.show();
      this.setCdTipoCard(tipoCard.value);
      this.setFilterModalPeriodo();
    } else {
      this.utilities.showAlert('alertError', 'Tipo de Cardápio não selecionado', 'warning');
    }
  }
  
  public setFilterTpCardapio({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
  
    component.datasource.filter = conditions;
    component.datasource.get().then((response: IDictionary) => {
      const filterData = (this.elaboracaoCardapio.getFilterData) ? this.elaboracaoCardapio.getFilterData : false;
      
      if (response.length > 0 && filterData) {
        const tipoCardFilter = filterData.typeMenu;
        
        this.dtIniCard = filterData.initialDate;
        this.dtIniPeriodo = filterData.initialDate;
        this.dtFinCard = filterData.finalDate;
        this.dtFinPeriodo = filterData.finalDate;
        this.descCard = tipoCardFilter?.DESCRICAO;
        
        component.setValue(tipoCardFilter?.CDTIPOCARD);
      }
    });
  }

  public setFilterServ({ event, component }: IEventParam<SelectMultiple>) {
    const cdtipoCard = Metadata.getInstance<Select>('TIPOCARD');
    let valueCard;
    
    if (cdtipoCard.returnObject === true && cdtipoCard.value !== null) valueCard = cdtipoCard.value.CDTIPOCARD;
    else valueCard = cdtipoCard.value;
    
    const conditions = [
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: cdtipoCard.value ? valueCard : '',
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    if (event?.type === 'click') {
      component.datasource.filter = conditions;
      component.datasource.get();
    } else {
      const filterData = (this.elaboracaoCardapio.getFilterData) ? this.elaboracaoCardapio.getFilterData : false;
      conditions[0].value = filterData.typeMenu?.CDTIPOCARD;
      component.datasource.filter = conditions;
      
      component.datasource.get().then((response: IDictionary) => {
        if (response.length > 0) component.setValue([filterData.service?.CDSERVFILI]);
      });
    }
  }
  
  public setFilterAgregTpCard({ component, event }: IEventParam<Select>) {
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD');

    let tpCardValue = '';
    if (tipoCard.returnObject) tpCardValue = tipoCard.value ? tipoCard.value.CDTIPOCARD : 'T';
    else tpCardValue = tipoCard.value ? tipoCard.value : 'T';
    
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: 'T',
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: tpCardValue,
      },
    ];
    
    component.datasource.filter = conditions;
    
    if (event?.type === 'click') {
      component.datasource.get();
    } else {
      component.datasource.get().then((response: IDictionary) => {
        const filterData = (this.elaboracaoCardapio.getFilterData) ? this.elaboracaoCardapio.getFilterData : false;
        
        if (response.length > 0) component.setValue(filterData.aggregated ? filterData.aggregated?.CDFILIAL : '');
      });
    }
  }
}
