import {
  Modal, ModalService, Form,
  Grid, IButtonEvent, IGridEvent,
  Text, TextInput, DialogService,
  Button, Select, Tabs, LoadingService, SelectTree, Checkbox,
} from '@zeedhi/common';
import {
  Config, Loader, Metadata, Http,
  IDictionary,
  IEventParam, I18n, dayjs as moment,
  IDatasource, DatasourceFactory,
} from '@zeedhi/core';
import { PmeMask } from '@/config';
import { Cardapio } from '../../../components/cardapio/common/Cardapio';

export class Pla00115DadosDoPrato {
  private dadosPrato!: Modal;
    
  private selectedPlate!: IDictionary;
    
  private idErrorMessage!: string;
    
  private productError!: IDictionary;
    
  public showCost = false;
    
  private typePlate!: IDictionary;
    
  private appController = Loader.getInstance('AppController');
    
  private utilities = Loader.getInstance('Utilities');
    
  private showGroup = false;
    
  private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');

  private receita = Loader.getInstance('ReceitaController');
  
  // private dataDadosPrato: IDictionary[] = [];

  public selectedNmPlate = '';

  public selectedIdReceita = '';

  public selectedCusPerCapita = '';

  public showCusPerCapita = false;

  public showFooter = true;
    
  public async createModal() {
    if (!this.dadosPrato) {
      this.dadosPrato = await ModalService.createFromJsonPost('',
        { jsonPath: 'pme/cardapio/modals/pla00115_dados_do_prato' });
    } else {
      await this.dadosPrato.destroy();
      this.dadosPrato = await ModalService.createFromJsonPost('',
        { jsonPath: 'pme/cardapio/modals/pla00115_dados_do_prato' });
    }
  }
  
  public filterDadosDoPrato = {};
  
  public dataGridDadosdoPrato: IDatasource = [];
    
  public destroyModal(onInit?: unknown) {
    if (this.dadosPrato) {
      this.dadosPrato.destroy();
      this.dadosPrato = this.utilities.modalEmpty;
    } else {
      const modal = ModalService.getModal('pla00115_dados_do_prato');
      modal?.destroy();
      this.dadosPrato = this.utilities.modalEmpty;
    }
    this.showFooter = true;
      
    if (typeof onInit !== 'boolean') {
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      cardapio.setValueAction(false);
    }
  }
  
  public async setFilterGroup({ component }: IEventParam<SelectTree>) {
    if (component && component.datasource) {
      component.datasource.filter = {
        CDFILAUXPRAT: (this.elaboracaoCardapio.getFilterData.aggregated !== null)
          ? this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL
          : this.elaboracaoCardapio.getFilterData.cdfilcentpme,
        CDNVPRATO: (Config.masks.plateMask as PmeMask).nivelMax,
      };
      await component.datasource.get();
      component.createNodesFromDatasource();
    }
  }

  private setShowGroup() {
    this.showGroup = (this.elaboracaoCardapio.getFilterData.paramplaneja.IDCONSGRPRAT !== 'N');
  }
  
  get colsPlateGroupCol(): number {
    return (this.elaboracaoCardapio.getFilterData.paramplaneja.IDCONSGRPRAT !== 'N') ? 4 : 0;
  }
  
  get colsPlateFilterCol(): number {
    return (this.elaboracaoCardapio.getFilterData.paramplaneja.IDCONSGRPRAT !== 'N') ? 4 : 6;
  }
  
  get colsPlateTypeCol(): number {
    return (this.elaboracaoCardapio.getFilterData.paramplaneja.IDCONSGRPRAT !== 'N') ? 4 : 6;
  }
    
  public async showModal(typePlate: IDictionary) {
    if (this.dadosPrato) this.destroyModal(true);
    await this.createModal();
    this.dadosPrato.show();
    this.setShowGroup();
    this.typePlate = typePlate;
    this.showCost = this.elaboracaoCardapio.getFilterData.searchPlate;
  }
    
  public visibleComensais() {
    const formPlate = Metadata.getInstance<Form>('formPlate');
    const numberDiners = formPlate.children.find((name) => name.name === 'numberDiners');
    if (numberDiners) {
      if (this.elaboracaoCardapio.getFilterData.checkDinners === true) numberDiners.mask.decimalPlaces = 0;
    }
  }
    
  public fullscreen({ component }: IButtonEvent) {
    const modal: IDictionary = Metadata.getInstance('modalHeaderDadosDoPrato');
    modal.parent.fullscreen = !modal.parent.fullscreen;
    component.iconName = (modal.parent.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
    const gridPlate = Metadata.getInstance<Grid>('gridPlate');
    gridPlate.maxHeight = (modal.parent.fullscreen) ? '65vh' : '50vh';
  }
    
  public async filterDataPlate() {
    const gridPlate = Metadata.getInstance<Grid>('gridPlate');
    gridPlate.datasource.loading = true;
    const route = (this.showCost) ? '/find/dadosPratoCusto' : '/find/dadosPrato';
    // const route = (this.showCost) ? '/find/dadosDoPratoCusto' : '/find/dadosDoPrato';
    const filter = this.getFilterData();
    const selectGroup = Metadata.getInstance<Select>('selectGroup').value;
    this.utilities.onMountedFilterFind(filter, 'P_CDPRATO', '=', selectGroup || null);
    const row = { ...gridPlate.datasource.currentRow };
    if (this.showCost) {
      const baseCost = this.elaboracaoCardapio.getFilterData.baseCost;
      const filial = filter.find((parameter: IDictionary) => parameter.name === 'P_CDFILICUSTO');
      if (baseCost === true) filial.value = this.appController.getCdCentral();
      else if (this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL !== null && baseCost === false) {
        filial.value = this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL;
      }
    }
      
    (gridPlate.datasource as IDictionary).route = route;
    gridPlate.datasource.filter = filter;
    
    Metadata.getInstance<Text>('valueTypePlate').text = this.typePlate.NMTIPOPRAT;
    Metadata.getInstance<Text>('valueData').text = moment(
      this.typePlate.DTCARDIAPME, 'DD/MM/YYYY',
    ).format(this.utilities.getFormatLocaleFullDataWithWeek());

    setTimeout(() => {
      this.utilities.focusOnField('gridPlateSearch');
    }, 1);
    
    // if (this.dataDadosPrato.length === 0 || row.CDPRATO) {
    // Http.post(route, { requestType: 'FilterData', filter }).then((response) => {
    // const datasource = response.data.data;
    const comensais = gridPlate.datasource.filter.find((data: IDictionary) => data.name === 'P_COMENSAIS');
    let selecionado: IDictionary;
    if (this.typePlate) {
      const selecionadoFilter = this.getFilterData();
      const cloneDs = gridPlate.datasource.clone();
      const filialAuxPrato = (this.elaboracaoCardapio.getFilterData.aggregated !== null)
        ? this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL
        : this.elaboracaoCardapio.getFilterData.cdfilcentpme;
      this.utilities.onMountedFilterFind(selecionadoFilter, 'P_PRATOSELECIONADO', '=',
        (this.typePlate?.CDPRATO === 'undefined') ? '' : this.typePlate.CDPRATO);
      this.utilities.onMountedFilterFind(selecionadoFilter, 'P_CDFILAUXPRATSELECIONADO', '=', filialAuxPrato);
      cloneDs.route = (this.showCost) ? '/find/pratoSelecionadoCusto' : '/find/pratoSelecionado';
      cloneDs.filter = selecionadoFilter;
      const pratoSelecionadoDs = DatasourceFactory.factory(cloneDs);
      selecionado = await pratoSelecionadoDs.get();
    } else {
      selecionado = {};
    }
    
    if (JSON.stringify(this.filterComparation(this.filterDadosDoPrato))
      !== JSON.stringify(this.filterComparation(gridPlate.datasource.filter))) {
      gridPlate.datasource.get().then(() => {
        let datasource: IDictionary[];
        if (selecionado.length > 0) {
          selecionado = selecionado[0];
          const data = gridPlate.datasource.data.filter(
            (el: IDictionary) => !(el.CDPRATO === selecionado.CDPRATO && el.CDFILAUXPRAT === selecionado.CDFILAUXPRAT),
          );
        
          datasource = [selecionado, ...data];
        } else {
          datasource = gridPlate.datasource.data;
        }
        datasource.forEach((collumn: IDictionary) => {
          const date = moment(collumn.DTULTATURECE, 'DD/MM/YYYY');
          collumn.DTULTATURECE = date.format(this.utilities.getFormatLocale());
        });
        this.dataGridDadosdoPrato = {
          datasource: gridPlate.datasource,
          data: [...gridPlate.datasource.data],
        };
        gridPlate.datasource.data = datasource;
        
        if (!row.CDPRATO) this.selectFirstData(gridPlate);
        else {
          const filteredRow = gridPlate.datasource.data.filter((e: IDictionary) => e.CDPRATO === row.CDPRATO);
          if (filteredRow.length > 0) gridPlate.datasource.currentRow = filteredRow[0];
          else this.selectFirstData(gridPlate);
        }
        this.filterDadosDoPrato = gridPlate.datasource.filter;
        
        // this.dataDadosPrato = datasource;
      });
    } else {
      gridPlate.datasource = this.dataGridDadosdoPrato.datasource;
      gridPlate.datasource.data = this.dataGridDadosdoPrato?.data || [];
      let datasource: IDictionary[];
      if (selecionado.length > 0) {
        selecionado = selecionado[0];
        const data = gridPlate.datasource.data.filter(
          (el: IDictionary) => !(el.CDPRATO === selecionado.CDPRATO && el.CDFILAUXPRAT === selecionado.CDFILAUXPRAT),
        );
      
        datasource = [selecionado, ...data];
      } else {
        datasource = gridPlate.datasource.data;
      }
      datasource.forEach((collumn: IDictionary) => {
        const date = moment(collumn.DTULTATURECE, 'DD/MM/YYYY');
        collumn.DTULTATURECE = date.format(this.utilities.getFormatLocale());
        if (collumn.COMENSAL) {
          collumn.COMENSAL = comensais.value;
          collumn.PERCAPLIC = comensais.value + ((comensais.value / 100) * collumn.PERACEIT);
        }
      });
      
      gridPlate.datasource.data = datasource;
        
      if (!row.CDPRATO) this.selectFirstData(gridPlate);
      else {
        const filteredRow = gridPlate.datasource.data.filter((e: IDictionary) => e.CDPRATO === row.CDPRATO);
        if (filteredRow.length > 0) gridPlate.datasource.currentRow = filteredRow[0];
        else this.selectFirstData(gridPlate);
      }
    }
    // Metadata.getInstance<IDictionary>('titleHeader').autofocus = true;
    /* } else {
      gridPlate.datasource.data = this.dataDadosPrato;
      this.selectFirstData(gridPlate);
    } */
  }
    
  private getFilterData() {
    const filter: IDictionary = [];
    // const selectGroup = Metadata.getInstance<Select>('selectGroup').value;
    const filialAuxPrato = (this.elaboracaoCardapio.getFilterData.aggregated !== null)
      ? this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL
      : this.elaboracaoCardapio.getFilterData.cdfilcentpme;
      
    // this.utilities.onMountedFilterFind(filter, 'P_CDPRATO', '=', selectGroup || null);
    // this.utilities.onMountedFilterFind(filter, 'P_PRATOSELECIONADO', '=',
    // (this.typePlate?.CDPRATO === 'undefined') ? '' : this.typePlate.CDPRATO);
    // this.utilities.onMountedFilterFind(filter, 'P_CDFILAUXPRATSELECIONADO', '=', filialAuxPrato);
    this.utilities.onMountedFilterFind(filter, 'P_CDFILIAL', '=', filialAuxPrato);
    this.utilities.onMountedFilterFind(filter, 'P_CDFILIAL_PERACEIT', '=', filialAuxPrato);
    this.utilities.onMountedFilterFind(filter, 'P_CDFILCENTPME', '=', this.elaboracaoCardapio.getFilterData.cdfilcentpme);
    this.utilities.onMountedFilterFind(
      filter, 'P_CDSERVFILI', '=',
      this.elaboracaoCardapio.getFilterData.service.CDSERVFILI,
    );
    this.utilities.onMountedFilterFind(
      filter, 'P_CDTIPOCARD', '=',
      this.elaboracaoCardapio.getFilterData.typeMenu.CDTIPOCARD,
    );
    this.utilities.onMountedFilterFind(filter, 'P_CDFILAUXPRAT', '=', filialAuxPrato);
    this.utilities.onMountedFilterFind(filter, 'P_COMENSAIS', '=', this.typePlate.QTCOMESTCPME);
    this.utilities.onMountedFilterFind(filter, 'P_NIVEL', '=', (Config.masks.plateMask as PmeMask).nivelMax);
    
    if (this.showCost) {
      this.utilities.onMountedFilterFind(filter, 'P_CDFILICUSTO', '=', filialAuxPrato);
      
      this.utilities.onMountedFilterFind(filter, 'P_NRTIPOCUSTO', '=', this.elaboracaoCardapio.getFilterData.planned);
      this.utilities.onMountedFilterFind(
        filter, 'P_DTCUSTOPROD', '=',
        (this.elaboracaoCardapio.getFilterData.radioSelect === 'R')
          ? this.elaboracaoCardapio.getFilterData.referenceDate
          : this.typePlate.DTCARDIAPME,
      );
    }
      
    return filter;
  }
    
  private selectFirstData(gridPlate: Grid) {
    const selectRow = (this.typePlate.CDPRATO === null)
      ? undefined
      : gridPlate.datasource.data.filter((data: IDictionary) => data.CDPRATO === this.typePlate.CDPRATO)[0];
    gridPlate.datasource.currentRow = (selectRow !== undefined) ? selectRow : gridPlate.datasource.data[0];
    this.selectedPlate = gridPlate.datasource.currentRow;
    this.receita.selectedPlate = gridPlate.datasource.currentRow;
    this.setPlateInfo();
    this.setValueDinersChange();
  }
    
  private getPerCons(diners: string): number {
    let VRPECONSCPME = 0;
    if (this.elaboracaoCardapio.getFilterData.IDPERCCOME === '%') {
      VRPECONSCPME = (this.typePlate.VRPECONSCPME === null) ? 100 : this.typePlate.VRPECONSCPME;
    } else {
      VRPECONSCPME = (this.typePlate.VRPECONSCPME === null && this.typePlate.QTCOMESTCPME === 0)
        ? this.typePlate.QTCOMESTCPME
        : ((this.typePlate.VRPECONSCPME / 100) * this.typePlate.QTCOMESTCPME);
    }
      
    return (VRPECONSCPME === 0 && diners !== undefined) ? parseInt(diners, 10) : VRPECONSCPME;
  }

  public selectPlate() {
    if (document.getElementsByName('gridPlate').length > 0) {
      const gridPlate: Grid = Metadata.getInstance('gridPlate');
      const row = gridPlate.datasource.currentRow;
      if (row.IDRECEITA) {
        this.selectedPlate = row;
        this.setPlateInfo();
          
        this.receita.selectedPlate = row;
        this.mountFilter(row);
        if (row.IDATIVO !== 'N') {
          Http.post('/recipeValidate', { requestType: 'Row', row }).then((response) => {
            const ativpr = response.data.dataset.recipe.filter((data: IDictionary) => data.IDPRODATIVPR === 'N');
            const ativ = response.data.dataset.recipe.filter((data: IDictionary) => data.IDPRODATIV === 'I');
            if (ativpr.length > 0) {
              this.selectedPlate = {};
              this.idErrorMessage = 'N';
              this.productError = ativpr;
            } else if (ativ.length > 0) {
              this.selectedPlate = {};
              this.idErrorMessage = 'I';
              this.productError = ativ;
            }
          });
        } else this.utilities.showAlert('inativo', 'Prato ou receita utilizada inativa.', 'warning');
      } else this.selectedPlate = {};
    }
  }

  private setPlateInfo() {
    if (this.selectedPlate.IDRECEITA === 'F') this.selectedIdReceita = 'Local';
    else if (this.selectedPlate.IDRECEITA === 'S') this.selectedIdReceita = 'Serviço';
    else if (this.selectedPlate.IDRECEITA === 'T') this.selectedIdReceita = 'Tipo de Cardápio';
    else if (this.selectedPlate.IDRECEITA === 'P') this.selectedIdReceita = 'Padrão';
    else if (this.selectedPlate.IDRECEITA === 'N') this.selectedIdReceita = 'Nenhum';

    this.selectedNmPlate = this.selectedPlate.NMPRATO;
      
    const currency = localStorage.getItem('CURRENCY');
    let value = '';
    if (!this.selectedPlate.CUSTO || this.selectedPlate.CUSTO === 0) value = '0,00';
    else value = this.utilities.formatValuesToFixed(this.selectedPlate.CUSTO, 2);
      
    this.selectedCusPerCapita = `${currency}${value}`;

    this.showCusPerCapita = this.elaboracaoCardapio.getFilterData.searchPlate || false;
  }

  public attPlateInfo() {
    const gridPlate = Metadata.getInstance<Grid>('gridPlate');
    gridPlate.datasource.get().then((response: IDictionary) => {
      const data = response.filter((e: IDictionary) => e.CDPRATO === this.selectedPlate.CDPRATO);
      if (data.length > 0) gridPlate.datasource.currentRow = data[0];
    });
  }

  public setFunctionRecipe() {
    this.receita.eventRecipeChange = this.attPlateInfo;
  }
    
  private mountFilter(row: IDictionary) {
    row.CDNVPRODUTO = (Config.masks.productMask as PmeMask).nivelMax;
    row.CDFILIAL = this.typePlate.CDFILIAL;
    row.CDSERVFILI = this.elaboracaoCardapio.getFilterData.service.CDSERVFILI;
    row.CDFILCENTPME = this.elaboracaoCardapio.getFilterData.cdfilcentpme;
    row.CDTIPOCARD = this.elaboracaoCardapio.getFilterData.typeMenu.CDTIPOCARD;
  }
    
  private validReceita(): boolean {
    if (this.selectedPlate.IDRECEITA === 'N') {
      this.utilities.showAlert('saveWithoutPlate', 'Operação bloqueada! Prato não possui receita.', 'error');
      return false;
    }
      
    if (this.selectedPlate.IDRECEITA === 'P') {
      // if (this.selectedPlate.IDUTRPADCA === 'S') return true;
      if (this.elaboracaoCardapio.utilRecPadr === 'S') return true;
        
      this.utilities.showAlert('saveWithoutPlate', 'Operação bloqueada! Prato não possui receita.Revisada', 'error');
      return false;
    }
      
    return true;
  }
    
  public saveModal({ component }: IEventParam<Button>) {
    const formModal = Metadata.getInstance<Form>('formPlate');
    if (Object.keys(this.selectedPlate).length > 0) { // Verifica se possui dados para inserção
      if (formModal.value.numberDiners > 0) { // Verifica se o numero de comensais foi inserido
        formModal.value.selectedPlate = this.selectedPlate; // Prato que foi selecionado para ser inserido
        // Dados onde serão inseridos os pratos
        formModal.value.plateData = formModal.value.checkboxDays.length > 0
          ? this.getContentForDay(formModal.value)
          : [[this.typePlate]];
        const row = formModal.value;
        if (this.validReceita()) {
          this.calcPercent(row);
          LoadingService.show(' ');
          
          if (this.selectedPlate.IDATIVO !== 'N') {
            this.callSavePlateData(row).then((savePlateData) => {
              if (savePlateData) {
                this.updateRegisterCard(formModal.value);
                this.destroyModal();
                this.utilities.showAlert('plateSaved', 'Salvo com sucesso!', 'success');
              }
            }).finally(() => {
              LoadingService.hide();
            });
          } else {
            this.utilities.showAlert('saveWithoutPlate', 'É necessário selecionar um prato ativo.', 'error');
            component.disabled = false;
          }
        } else {
          component.disabled = false;
        }
      } else {
        this.utilities.showAlert('nullDiner', 'Campo de comensais não pode ser menor ou igual a zero.', 'error');
      }
    } else {
      this.errorSaveModal();
    }
  }
    
  public callSavePlateData(row: IDictionary) {
    // const self = this;
    return Http.post('plate/new', { requestType: 'Row', row }).then((response) => {
      const acompanhamentoCusto = Loader.getInstance('Pla00106AcompanhamentoCusto');
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      const daySelected = cardapio.getCurrentSelectedRegister()?.DTCARDIAPME;
      
      acompanhamentoCusto.autoUpdateAcomp(daySelected);
      const plateData = response.data.dataset.plateSaved[0];
      return plateData;
    });
  }
    
  private updateRegisterCard(formModal: IDictionary) {
    if (formModal.checkboxDays.length === 0) {
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      const cardData = cardapio.dates.find((data: IDictionary) => data.DTCARDIAPME === this.typePlate.DTCARDIAPME);
      this.typePlate.CDPRATO = formModal.selectedPlate.CDPRATO;
      this.typePlate.NMPRATO = formModal.selectedPlate.NMPRATO;
      this.typePlate.CDFILAUXPRAT = formModal.selectedPlate.CDFILAUXPRAT;
      this.typePlate.IDCARDESPEC = (this.typePlate.filterData.aggregated === null) ? 'N' : 'S';
      cardData.IDCARDESPEC = (this.typePlate.filterData.aggregated === null) ? 'N' : 'S';
      // cardapio.forceUpdateInCard(); /* comentado pela rotina analise de cores */
      this.elaboracaoCardapio.reloadCard();
    } else {
      this.elaboracaoCardapio.reloadCard();
    }
    
    if (this.elaboracaoCardapio.showColorsAnalysis) this.elaboracaoCardapio.disableColorAnalysis();
  }
    
  private errorSaveModal() {
    if (this.idErrorMessage === 'N') {
      this.utilities.showAlert('inativo',
        'Operação bloqueada! Prato não pode conter produto inativo na receita.',
        'error');
      this.showDialog();
    } else if (this.idErrorMessage === 'I') {
      this.utilities.showAlert('inativo',
        'Operação bloqueada! Prato não pode conter produto inativo para a unidade.',
        'error');
      this.showDialog();
    } else if (!(this.selectedPlate.length > 0)) {
      this.utilities.showAlert('saveWithoutPlate', 'É necessário selecionar um prato ativo.', 'error');
    } else {
      this.utilities.showAlert('withoutRecipe', 'Operação Bloqueada! Sem receita cadastrada.', 'error');
    }
  }
    
  private calcPercent(row: IDictionary) {
    // Se for %, calcula a porcentagem inserida.
    let percent: any = 0;
    if (this.typePlate.QTCOMESTCPME !== 0) {
      const calc = (row.numberDiners * 100) / this.typePlate.QTCOMESTCPME;
      percent = (this.elaboracaoCardapio.getFilterData.IDPERCCOME === 'C')
        ? this.utilities.formatValuesToFixed(calc, 2)
        : row.numberDiners;
    }
    let plate: IDictionary = {};
      
    row.plateData.forEach((e: IDictionary) => {
      plate = e[0];
      plate.WEEKDAY = moment(plate.DTCARDIAPME, 'DD/MM/YYYY').day();
      plate.filterData = this.elaboracaoCardapio.getFilterData;
      plate.IDCARDESPEC = this.typePlate.IDCARDESPEC;
      plate.CDFILCENTPME = this.elaboracaoCardapio.getFilterData.cdfilcentpme;
      plate.NRCARDPME = this.typePlate.NRCARDPME;
      plate.CDSERVFILI = this.elaboracaoCardapio.getFilterData.service.CDSERVFILI;
      // plate.VRPECONSCPME = (plate.DTCARDIAPME === this.typePlate.DTCARDIAPME) ? percent : 100;
      plate.VRPECONSCPME = percent === 0 ? 100 : parseFloat(percent);
    });
  }
    
  private getContentForDay(dataForm: IDictionary) {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    const datesInDay: IDictionary = [];
    const plateInDay: IDictionary = [];
    dataForm.checkboxDays.forEach((weekday: string) => {
      const daySelected = cardapio.getCurrentSelectedRegister()?.DTCARDIAPME;
      const dataSet = cardapio.dates.filter(
        (dates: IDictionary) => dates.WEEKDAY === parseInt(weekday, 10)
          && dates.DTCARDIAPME !== daySelected && dates.DTCARDIAPME > daySelected,
      );
      dataSet.forEach((date: IDictionary) => datesInDay.push(date.DTCARDIAPME));
    });
      
    plateInDay.push([this.typePlate]);
      
    cardapio.menuContent.forEach((collumn: IDictionary) => {
      datesInDay.forEach((day: string) => {
        const plate = collumn.filter(
          (plates: IDictionary) => plates.CDTIPOPRAT === this.typePlate.CDTIPOPRAT
            && plates.DTCARDIAPME === day && plates.NRSEQCNFCPME === this.typePlate.NRSEQCNFCPME,
        );
        if (plate.length > 0) plateInDay.push(plate);
      });
    });
      
    return plateInDay;
  }
    
  private showDialog() {
    DialogService.show({
      name: 'pla00115_produtosInativos',
      type: 'error',
      title: 'Produtos Inativos',
      text: this.textError(),
      buttons: [
        {
          name: 'ok',
          component: 'ZdButton',
          label: 'OK',
          events: { click: () => DialogService.hide() },
          outline: true,
        },
      ],
    });
  }
    
  // Monta a mensagem que será exibida no texto do dialogo quando existe produtos inativos.
  private textError() {
    let message = '';
    if (this.productError.length > 0) {
      this.productError.forEach((product: IDictionary) => {
        message += product.DESCRICAO;
        message += '<br>';
      });
    }
    return message;
  }
    
  public modifyColor({ row }: IGridEvent) {
    if (row.IDATIVO === 'N') return 'inactive-plate';
    return '';
  }
    
  public setTab({ component }: IEventParam<Tabs>) {
    this.showFooter = true;
    if (component.activeTab === 1) this.setDataSourceReceita();
    else if (component.activeTab === 2) {
      this.showFooter = false;
      this.configRevisao();
    }
  }

  private configRevisao() {
    this.receita.tpCard = this.elaboracaoCardapio.getFilterData.typeMenu;
    this.receita.branch = this.elaboracaoCardapio.getFilterData.aggregated
      || { CDFILIAL: this.appController.getCdCentral(), NMFILIAL: this.appController.getNmCentral() };
    this.receita.service = this.elaboracaoCardapio.getFilterData.service;
    this.receita.propChange = false;
    this.receita.setTabPlate();
    this.receita.setFilterGrids();
  }

  public setTabPrato() {
    const tabFilha = Metadata.getInstance<Tabs>('tabsGrid');
    tabFilha.activeTab = 0;
    this.showFooter = true;
  }
  
  public setSelectedDays({ component }: { component: { value: number[] } }): void {
    const daysSelected = Metadata.getInstance<Checkbox>('checkboxDays');
    const todos = component.value;
    const weekdays: number[] = [0, 1, 2, 3, 4, 5, 6];
    // Se 9 estiver selecionado, adicionar os dias da semana (0 a 6)
    if (todos) {
      // Adicionar os dias da semana apenas se não estiverem já presentes
      weekdays.forEach((day: number) => {
        if (!daysSelected.value.includes(day)) {
          daysSelected.value.push(day);
        }
      });
    } else {
      daysSelected.value = [];
    }
  }

  public setDataSourceReceita() {
    const dtCusto = this.elaboracaoCardapio.getFilterData.searchPlate;
    const referenceDate = this.elaboracaoCardapio.getFilterData.referenceDate;
    const gridPlate: Grid = Metadata.getInstance('gridPlate');
    const row = gridPlate.datasource.currentRow;
    this.mountFilter(row);
    if (document.getElementsByName('gridReceitaUtilizada').length > 0) {
      const gridReceitaUtilizada: Grid = Metadata.getInstance('gridReceitaUtilizada');
      gridReceitaUtilizada.datasource.loading = true;
      if (row.IDATIVO !== 'N') {
        if (this.showCost || dtCusto) {
          const cdFiliCusto = this.elaboracaoCardapio.getFilterData.baseCost;
          if (cdFiliCusto) {
            row.CDFILICUSTO = this.elaboracaoCardapio.getFilterData.cdfilcentpme;
          } else {
            row.CDFILICUSTO = (this.elaboracaoCardapio.getFilterData.aggregated === null)
              ? this.elaboracaoCardapio.getFilterData.cdfilcentpme
              : this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL;
          }
            
          row.DTCUSTO = referenceDate ? this.elaboracaoCardapio.getFilterData.referenceDate : this.typePlate.DTCARDIAPME;
          row.NRTIPOCUSTO = this.elaboracaoCardapio.getFilterData.planned;
          // row.CDPRATO = this.typePlate.CDPRATO;
          // row.CDFILAUXPRAT = this.typePlate.CDFILAUXPRAT;
          // row.IDRECEITA = this.selectedIdReceita;
          Http.post('receitaUtilizadaCusto', { requestType: 'Row', row }).then((response) => {
            const result = response.data.dataset.result;
            gridReceitaUtilizada.datasource.data = result;
            gridReceitaUtilizada.datasource.loading = false;
          });
        } else {
          Http.post('/receitaUtilizada', { requestType: 'Row', row }).then((response) => {
            const result = response.data.dataset.result;
            gridReceitaUtilizada.datasource.data = result;
            gridReceitaUtilizada.datasource.loading = false;
          });
        }
      } else {
        gridReceitaUtilizada.datasource.data = [];
        gridReceitaUtilizada.datasource.loading = false;
      }
    }
  }
    
  private setValueDinersChange() {
    const gridPlate = Metadata.getInstance<Grid>('gridPlate');
    let comensal = gridPlate.datasource.currentRow.COMENSAL;
    const hasPlate = this.typePlate.CDPRATO;
    
    // Considera a currentRow do datasource como a primeira linha
    // Quando o datasource é recarregado e fica sem nenhuma linha, perde-se a referência da currentRow
    if (!comensal && gridPlate.datasource.data.length > 0) {
      comensal = gridPlate.datasource.data[0].COMENSAL;
    }

    const percAplic = gridPlate.datasource.currentRow.PERCAPLIC;
    const perAceit = gridPlate.datasource.currentRow.PERACEIT;
    const formModal = Metadata.getInstance<Form>('formPlate');
    const filialFilterFlag = (this.elaboracaoCardapio.getFilterData.aggregated !== null);
    formModal.value.numberDiners = this.getPerCons(comensal);

    formModal.getChildInstance<TextInput>('numberDiners').label = (this.elaboracaoCardapio.getFilterData.IDPERCCOME === '%')
      ? 'Comensais (%)'
      : 'Comensais';
    
    if (percAplic > 0 && percAplic !== comensal && filialFilterFlag && hasPlate == null) {
      // altera label adicionando percentual de adesão apenas quando altera o valor do comensal
      if (perAceit !== 0 && perAceit !== null && perAceit !== undefined) {
        const label = formModal.getChildInstance<TextInput>('numberDiners').label;
        formModal.getChildInstance<TextInput>('numberDiners').label = I18n.translate(label)
          + I18n.translate(' (Perc. Adesão Aplicado)');
      }
      if (this.elaboracaoCardapio.getFilterData.IDPERCCOME === '%') {
        formModal.value.numberDiners = (percAplic / comensal) * 100;
      } else {
        formModal.value.numberDiners = percAplic;
      }
    } else {
      const label = formModal.getChildInstance<TextInput>('numberDiners').label;
      formModal.getChildInstance<TextInput>('numberDiners').label = I18n.translate(label);
    }
  }

  public onChangePlate() {
    this.selectPlate();
    this.setValueDinersChange();
    if (Object.keys(this.selectedPlate).length > 0) {
      const tabFilha = Metadata.getInstance<Tabs>('tabsGrid');
      tabFilha.tabs[1].disabled = false;
      tabFilha.tabs[2].disabled = false;
    }
  }
    
  public pratoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDPRATO',
        mask: (Config.masks.plateMask as PmeMask).pattern,
      },
      'NMPRATO',
    ],
    formatterDataText: [
      {
        name: 'CDPRATO',
        mask: (Config.masks.plateMask as PmeMask).pattern,
      },
      'NMPRATO',
    ],
    dataValueOut: [
      {
        column: 'CDPRATO',
        columnOnGrid: 'CDPRATO',
      },
      {
        column: 'NMPRATO',
        columnOnGrid: 'NMPRATO',
      },
    ],
    dataValue: 'CDPRATO',
  };
    
  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
    ],
    dataValue: 'CDPRODUTO',
    dense: true,
    datasource: {
      uniqueKey: 'CDPRODUTO',
      route: '/prodUltimoNivel',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'Campo obrigatório',
      },
    },
  };
  
  public filterComparation(filter: IDictionary) {
    const newFilter: IDictionary = [];
    if (filter.length > 0) {
      filter.forEach((el: IDictionary) => {
        if (el.name !== 'P_COMENSAIS') {
          newFilter.push(el);
        }
      });
    }
    return newFilter;
  }
  
  public reloadDadosDoPrato() {
    this.filterDadosDoPrato = {};
    this.dataGridDadosdoPrato = {};
    this.filterDataPlate();
  }
  
  public rowClick({ event }: IEventParam<Grid>) {
    const buttonSave = Metadata.getInstance<IButtonEvent>('saveModal');
    this.utilities.doubleClick(
      () => this.saveModal(buttonSave),
      event,
    );
  }
}
