import {
  Http, IDictionary, Loader, IEventParam, Metadata, Config,
} from '@zeedhi/core';
import {
  Select, Form, DialogService, Button, SelectTree, LoadingService,
} from '@zeedhi/common';

export class ReportController {
  public appController = Loader.getInstance('AppController');
  
  public utilities = Loader.getInstance('Utilities');

  public generateReport(url: string, row: IDictionary, cdReport: string) {
    row.__report_name = cdReport;
    row.isUrl = false;
    return Http.post(url, { requestType: 'Row', row }).then((response) => {
      const uri = response.data.messages[0].message;
      this.openReport(uri);
    });
  }

  private openReport(uri: string) {
    const baseUrl = Config.endPoint;
    window.open(`${baseUrl}${uri}`);
  }

  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }

  public setFilterTpCardapio({ component }: IEventParam<Select>) {
    let central = this.appController.getCdCentral();
    
    let campoCentral = Metadata.getInstance<Select>('CENTRAL');
    if (campoCentral) central = campoCentral.value;
    
    if (component.name === 'TIPOCARDDEST') {
      campoCentral = Metadata.getInstance<Select>('CENTRALDESTINO');
      if (campoCentral) central = campoCentral.value;
    }
    
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: central,
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterUnid({ component }: IEventParam<Select>) {
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterAgregAgrup({ component }: IEventParam<Select>) {
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnid.value ? agrupUnid.value : 'T',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public setFilterAgreg({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: 'T',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public setFilterAgregTpCard({ component }: IEventParam<Select>) {
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD');

    let tpCardValue = '';
    if (tipoCard.returnObject) tpCardValue = tipoCard.value ? tipoCard.value.CDTIPOCARD : 'T';
    else tpCardValue = tipoCard.value ? tipoCard.value : 'T';
    
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: 'T',
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: tpCardValue,
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public setFilterServAgrupUnid({ component }: IEventParam<Select>) {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const unidadeValue = (unidade.value.length !== 0) ? unidade.value.join('_') : 'T';
    
    const grupserv = Metadata.getInstance<Select>('GRUPOSERVICO');
    const grupServValue = grupserv.value ? grupserv.value : 'T';
    
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    const agrupUnidValue = agrupUnid.value ? agrupUnid.value : 'T';

    const filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidadeValue,
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDGRUPSERV',
        operator: '=',
        value: grupServValue,
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnidValue,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }

  public setFilterServ({ component }: IEventParam<Select>) {
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD');
    let tpCardValue = '';
    if (tipoCard.returnObject) tpCardValue = tipoCard.value ? tipoCard.value.CDTIPOCARD : '';
    else if (Array.isArray(tipoCard.value)) tpCardValue = tipoCard.value.join('_');
    else tpCardValue = tipoCard.value ? tipoCard.value : '';

    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: tpCardValue,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }

  public clearFields({ component }: IEventParam<Select>) {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const servico = Metadata.getInstance<Select>('SERVICO');
    this.utilities.clearPeriodoParams();

    if (component.name === 'TIPOCARD' || component.name === 'GRUPFILI' || component.name === 'TIPOCARD_CARDPERIODO') {
      servico.setValue('');
      unidade.setValue('');
    } else if (component.name === 'UNIDADE' || component.name === 'GRUPOSERVICO') {
      servico.setValue('');
    } else unidade.setValue('');
  }
  
  public formIsValid(form: Form) {
    let valid = false;
    const row: IDictionary = form.value;
    if (this.utilities.centralIsValid() && form.validate()) {
      if (this.utilities.dateIsValid(row.DTDE, row.DTATE)) {
        valid = true;
      } else this.utilities.showAlert('alertError', 'Data Inicial deve ser menor que a data final.', 'error');
    }
    return valid;
  }
  
  public verifyProductIni({ component }: IEventParam<SelectTree>) {
    const productIni = Metadata.getInstance<SelectTree>('PRODINICIAL');
    const productFin = Metadata.getInstance<SelectTree>('PRODFINAL');
    
    if ((productIni.value && productFin.value && productIni.value.CDARVPROD > productFin.value.CDARVPROD)
      || (productIni.value > productFin.value)) {
      this.utilities.showAlert('Alert', 'Produto inicial maior que o final.', 'error');
      setTimeout(() => { component.setValue(null); }, 100);
    }
  }
  
  public controllProductIni() {
    const productFin = Metadata.getInstance<SelectTree>('PRODFINAL');
    if (productFin.value !== null) productFin.setValue(null);
  }

  public defaultTpCusto({ component }: IEventParam<Select>) {
    const language = this.utilities.getLanguage() as string;
    let idioma = null;
    if (language != null && language.includes('en')) idioma = 'I';
    else if (language != null && language.includes('es')) idioma = 'E';
    
    const conditions = [
      {
        name: 'STIDIOMA',
        operator: '=',
        value: idioma,
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get().then((response: IDictionary) => {
      if (response.length > 0) component.setValue(response[0]);
    });
  }
  
  public callGenerateReport(component: Button, row: IDictionary, reportName: string) {
    row.reportFormat = component.name;
    // this.utilities.activeLoadingButton(component);
    // grid.datasource.loading
    LoadingService.show(' ');
    this.generateReport(reportName, row, row.CD_REPORT_CODE).finally(() => {
      // this.utilities.disableLoadingButton(component);
      LoadingService.hide();
    });
    DialogService.hide();
  }
}
